@@ -2,7 +2,7 @@ package Form::Tiny;
 
 use v5.10;
 use warnings;
-use Carp qw(croak);
+use Carp qw(croak carp);
 use Types::Standard qw(Str);
 use Import::Into;
 
@@ -10,14 +10,14 @@ use Form::Tiny::Form;
 use Form::Tiny::Utils qw(trim :meta_handlers);
 require Moo;
 
-our $VERSION = '2.00';
+our $VERSION = '2.02';
 
 sub import
 {
 	my ($package, $caller) = (shift, scalar caller);
 
 	my @wanted = @_;
-	my @wanted_subs = qw(form_field form_cleaner form_hook);
+	my @wanted_subs = qw(form_field form_cleaner form_hook field_validator);
 	my @wanted_roles;
 
 	my %subs = %{$package->_generate_helpers($caller)};
@@ -55,20 +55,40 @@ sub _generate_helpers
 {
 	my ($package, $caller) = @_;
 
+	my $field_context;
+	my $use_context = sub {
+		if (@_ == 2) {
+			croak 'context using DSL keyword called without context'
+				unless defined $field_context;
+			unshift @_, $field_context;
+		}
+		return @_;
+	};
+
 	return {
 		form_field => sub {
-			$caller->form_meta->add_field(@_);
+			$field_context = $caller->form_meta->add_field(@_);
 		},
 		form_cleaner => sub {
+			$field_context = undef;
 			$caller->form_meta->add_hook(cleanup => @_);
 		},
 		form_hook => sub {
+			$field_context = undef;
 			$caller->form_meta->add_hook(@_);
 		},
 		form_filter => sub {
+			$field_context = undef;
 			$caller->form_meta->add_filter(@_);
 		},
+		field_filter => sub {
+			$caller->form_meta->add_field_filter($use_context->(@_));
+		},
+		field_validator => sub {
+			$caller->form_meta->add_field_validator($use_context->(@_));
+		},
 		form_trim_strings => sub {
+			$field_context = undef;
 			$caller->form_meta->add_filter(Str, \&trim);
 		},
 	};
@@ -89,7 +109,7 @@ sub _get_behaviors
 			roles => [qw(Form::Tiny::Meta::Strict)],
 		},
 		-filtered => {
-			subs => [qw(form_filter form_trim_strings)],
+			subs => [qw(form_filter field_filter form_trim_strings)],
 			roles => [qw(Form::Tiny::Meta::Filtered)],
 		},
 	};
@@ -110,11 +130,11 @@ Form::Tiny - Input validator implementation centered around Type::Tiny
 	use Form::Tiny -base;
 	use Types::Standard qw(Int);
 
-	form_filed 'my_field' => (
+	form_field 'my_field' => (
 		required => 1,
 	);
 
-	form_filed 'another_field' => (
+	form_field 'another_field' => (
 		type => Int,
 		default => sub { 0 },
 	);
@@ -171,7 +191,7 @@ Installed functions: C<form_field form_cleaner form_hook>
 
 =item * C<-nomoo>
 
-This flag stops Form::Tiny from importing Moo into your namespace. Unless you use a different class system (like L<Moose>) will have to declare your own constructor, which will need to call C<< $form->BUILD >>.
+This flag stops Form::Tiny from importing Moo into your namespace. Unless you use a different class system (like L<Moose>) will have to declare your own constructor.
 
 Installed functions: same as C<-base>
 
@@ -179,7 +199,7 @@ Installed functions: same as C<-base>
 
 This flag enables filters in your form.
 
-Installed functions: all of C<-base> plus C<form_filter form_trim_strings>
+Installed functions: all of C<-base> plus C<form_filter field_filter form_trim_strings>
 
 =item * C<-strict>
 
@@ -199,7 +219,7 @@ Installed functions: same as C<-base>
 
 This helper declares a new field for your form. Each style of calling this function should contain keys that meet the specification of L<Form::Tiny::FieldDefinition>, or an object of this class directly.
 
-In the first (hash) version, C<%arguments> need to be a plain hash (not a hashref) and should B<not> include the name in the hash, as it will be overriden by the first argument C<$name>.
+In the first (hash) version, C<%arguments> need to be a plain hash (not a hashref) and should B<not> include the name in the hash, as it will be overriden by the first argument C<$name>. This form also sets the context for the form being built: see L<Form::Tiny::Manual/"Context"> for details.
 
 In the second (coderef) version, C<$coderef> gets passed the form instance as its only argument and should return a hashref or a constructed object of L<Form::Tiny::FieldDefinition>. A hashref must contain a C<name>. Note that this creates I<dynamic field>, which will be resolved repeatedly during form validation. As such, it should not contain any randomness.
 
@@ -217,11 +237,24 @@ This creates a new hook for C<$stage>. Each stage may have multiple hooks and ea
 
 A shortcut for C<< form_hook cleanup => $coderef; >>.
 
+=head3 field_validator
+
+	field_validator $message => $coderef; # uses current context
+
+Adds an additional custom validator, ran after the type of the field is validated. C<$message> should be something that can present itself as a string. If for a given input parameter C<$coderef> returns false, that message will be added to form errors for that field. See L<Form::Tiny::Manual/"Additional validators"> for details.
+
 =head3 form_filter
 
 	form_filter $type, $coderef;
 
-C<$type> should be a Type::Tiny (or compatible) type check. For each input field that passes that check, C<$coderef> will be ran. See L<Form::Tiny::Manual/"Filters"> for details on filters.
+Filters the input value before the validation. C<$type> should be a Type::Tiny (or compatible) type check. For each input field that passes that check, C<$coderef> will be ran. See L<Form::Tiny::Manual/"Filters"> for details on filters.
+
+=head3 field_filter
+
+	field_filter $type, $coderef; # uses current context
+	field_filter $name => $type, $coderef;
+
+Same as C<form_filter>, but is narrowed down to a single form field identified by its name. Name can be omitted and the current context will be used. See L<Form::Tiny::Manual/"Context"> for details on context.
 
 =head3 form_trim_strings
 
