@@ -3,7 +3,7 @@ package Form::Tiny::FieldDefinition;
 use v5.10;
 use warnings;
 use Moo;
-use Types::Standard qw(Enum Bool HasMethods CodeRef InstanceOf);
+use Types::Standard qw(Enum Bool HasMethods CodeRef InstanceOf HashRef);
 use Types::Common::String qw(NonEmptySimpleStr);
 use Types::TypeTiny qw(StringLike);
 use Carp qw(croak);
@@ -16,7 +16,7 @@ use Form::Tiny::PathValue;
 
 use namespace::clean;
 
-our $VERSION = '2.01';
+our $VERSION = '2.02';
 
 has "name" => (
 	is => "ro",
@@ -45,6 +45,12 @@ has "type" => (
 	predicate => 1,
 );
 
+has 'addons' => (
+	is => 'rw',
+	isa => HashRef,
+	default => sub { {} },
+);
+
 has "coerce" => (
 	is => "ro",
 	isa => Bool | CodeRef,
@@ -178,22 +184,33 @@ sub validate
 {
 	my ($self, $form, $value) = @_;
 
-	# no validation if no type specified
-	return 1
-		if !$self->has_type;
+	my @errors;
 
-	my $valid;
-	my $error;
-	if ($self->has_message) {
-		$valid = $self->type->check($value);
-		$error = $self->message;
-	}
-	else {
-		$error = $self->type->validate($value);
-		$valid = !defined $error;
+	if ($self->has_type) {
+		if ($self->has_message) {
+			push @errors, $self->message
+				if !$self->type->check($value)
+		}
+		else {
+			my $error = $self->type->validate($value);
+			if (defined $error) {
+				push @errors, $error;
+			}
+		}
 	}
 
-	if (!$valid) {
+	if (@errors == 0) {
+		my $validators = $self->addons->{validators} // [];
+		for my $validator (@{$validators}) {
+			my ($message, $code) = @{$validator};
+
+			if (!$code->($value)) {
+				push @errors, $message;
+			}
+		}
+	}
+
+	for my $error (@errors) {
 		if ($self->is_subform && ref $error eq 'ARRAY') {
 			foreach my $exception (@$error) {
 				if (defined blessed $exception && $exception->isa("Form::Tiny::Error")) {
@@ -227,7 +244,7 @@ sub validate
 		}
 	}
 
-	return $valid;
+	return @errors == 0;
 }
 
 1;
@@ -290,6 +307,10 @@ The type attribute is where you can plug in a Type::Tiny type object. It has to
 
 B<predicate:> I<has_type>
 
+=head2 addons
+
+Hash reference for internal use only - readable and writable under the C<addons> method. If you need additional data for a field definition that will be used in metaclasses (while extending Form::Tiny), put it here.
+
 =head2 coerce
 
 Coercions take place just before the validation. By default, values are not coerced. Specifying value I<1> will cause the field to use coercions from the type object.
