@@ -4,7 +4,6 @@ use v5.10;
 use warnings;
 use Types::Standard qw(Maybe ArrayRef InstanceOf HashRef Bool);
 use Carp qw(croak);
-use Storable qw(dclone);
 use Scalar::Util qw(blessed);
 
 use Form::Tiny::PathValue;
@@ -12,7 +11,7 @@ use Form::Tiny::Error;
 use Form::Tiny::Utils qw(try get_package_form_meta);
 use Moo::Role;
 
-our $VERSION = '2.00';
+our $VERSION = '2.02';
 
 has "input" => (
 	is => "ro",
@@ -166,8 +165,6 @@ sub _validate
 
 	my $fields = $self->input;
 	my $err = try sub {
-		$fields = dclone($fields)
-			if ref $fields ne '';
 		$fields = $meta->run_hooks_for('reformat', $self, $fields);
 	};
 
@@ -260,6 +257,7 @@ sub add_error
 	}
 
 	push @{$self->errors}, $error;
+	$self->form_meta->run_hooks_for('after_error', $self, $error);
 	return $self;
 }
 
@@ -295,6 +293,14 @@ sub form_meta
 	return get_package_form_meta($package);
 }
 
+# This fixes form inheritance for other role systems than Moose
+around DOES => sub {
+	my ($orig, $self, @args) = @_;
+
+	return Moo::Role::does_role($self, @args)
+		|| $self->$orig(@args);
+};
+
 1;
 
 __END__
