@@ -5,14 +5,14 @@ use warnings;
 use Moo;
 use Types::Standard qw(ArrayRef HashRef InstanceOf Bool);
 use Scalar::Util qw(blessed);
-use Carp qw(croak);
+use Carp qw(croak carp);
 
 use Form::Tiny::FieldDefinitionBuilder;
 use Form::Tiny::Hook;
 
 use namespace::clean;
 
-our $VERSION = '2.00';
+our $VERSION = '2.02';
 
 has 'fields' => (
 	is => 'ro',
@@ -85,17 +85,26 @@ sub resolved_fields
 sub add_field
 {
 	my ($self, @parameters) = @_;
-	my $fields = $self->fields;
 
 	croak 'adding a form field requires at least one parameter'
 		unless scalar @parameters;
 
 	my $scalar_param = shift @parameters;
-	if (@parameters > 0 || ref $scalar_param eq '') {
-		$scalar_param = {@parameters, 'name', $scalar_param};
+	if (ref $scalar_param eq '') {
+		$scalar_param = {@parameters, name => $scalar_param};
 	}
 
-	push @{$fields}, Form::Tiny::FieldDefinitionBuilder->new(data => $scalar_param)->build;
+	my $builder = Form::Tiny::FieldDefinitionBuilder->new(data => $scalar_param)->build;
+	push @{$self->fields}, $builder;
+
+	return $builder;
+}
+
+sub add_field_validator
+{
+	my ($self, $field, $message, $code) = @_;
+
+	push @{$field->addons->{validators}}, [$message, $code];
 	return $self;
 }
 
